// Copyright (c) 2017, teja. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';
import 'package:mongo_dart/mongo_dart.dart' as mgo;
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_serializer/serializer.dart';

class MgoUserManagerEmail<ModelType extends UserEmail>
    implements AuthModelManager {
  final String collection;

  final mgo.Db db;

  final mgo.DbCollection _coll;

  final Serializer<ModelType> serializer;

  final Hasher hasher;

  MgoUserManagerEmail(this.db, this.collection, this.serializer,
      {Hasher hasher})
      : _coll = db.collection(collection),
        hasher = hasher ?? const NoHasher();

  Future<ModelType> fetchModelByAuthorizationId(String userId) async {
    Map map = await _coll.findOne(mgo.where.id(mgo.ObjectId.parse(userId)));
    return serializer.fromMap(map);
  }

  Future<ModelType> fetchModelByAuthenticationId(String email) async {
    Map map = await _coll.findOne(mgo.where.eq("email", email));
    return serializer.fromMap(map);
  }

  /// Checks password [keyword] for given userId [authId] and
  /// returns the user [UserModel] if passwords match. Returns [null] if the
  /// password doesn't match or user with given authId is not found.
  Future<ModelType> authenticate(String userId, String password) async {
    ModelType model = await fetchModelByAuthenticationId(userId);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.password)) {
      return null;
    }

    return model;
  }
}

class MgoUserManagerUsername<ModelType extends UserUsername>
    implements AuthModelManager {
  final String collection;

  final mgo.Db db;

  final mgo.DbCollection _coll;

  final Serializer<ModelType> serializer;

  final Hasher hasher;

  MgoUserManagerUsername(this.db, this.collection, this.serializer,
      {Hasher hasher})
      : _coll = db.collection(collection),
        hasher = hasher ?? const NoHasher();

  Future<ModelType> fetchModelByAuthorizationId(String userId) async {
    Map map = await _coll.findOne(mgo.where.id(mgo.ObjectId.parse(userId)));
    return serializer.fromMap(map);
  }

  Future<ModelType> fetchModelByAuthenticationId(String username) async {
    Map map = await _coll.findOne(mgo.where.eq("username", username));
    return serializer.fromMap(map);
  }

  /// Checks password [keyword] for given userId [authId] and
  /// returns the user [UserModel] if passwords match. Returns [null] if the
  /// password doesn't match or user with given authId is not found.
  Future<ModelType> authenticate(String userId, String password) async {
    ModelType model = await fetchModelByAuthenticationId(userId);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.password)) {
      return null;
    }

    return model;
  }
}

class MgoUserManagerUsernameEmail<ModelType extends UserUsernameEmail>
    implements AuthModelManager {
  final String collection;

  final mgo.Db db;

  final mgo.DbCollection _coll;

  final Serializer<ModelType> serializer;

  final Hasher hasher;

  MgoUserManagerUsernameEmail(this.db, this.collection, this.serializer,
      {Hasher hasher})
      : _coll = db.collection(collection),
        hasher = hasher ?? const NoHasher();

  Future<ModelType> fetchModelByAuthorizationId(String userId) async {
    Map map = await _coll.findOne(mgo.where.id(mgo.ObjectId.parse(userId)));
    return serializer.fromMap(map);
  }

  Future<ModelType> fetchModelByAuthenticationId(String username) async {
    Map map = await _coll.findOne(mgo.where.eq("username", username));

    if (map != null) {
      return serializer.fromMap(map);
    }

    map = await _coll.findOne(mgo.where.eq("email", username));

    return serializer.fromMap(map);
  }

  /// Checks password [keyword] for given userId [authId] and
  /// returns the user [UserModel] if passwords match. Returns [null] if the
  /// password doesn't match or user with given authId is not found.
  Future<ModelType> authenticate(String userId, String password) async {
    ModelType model = await fetchModelByAuthenticationId(userId);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.password)) {
      return null;
    }

    return model;
  }
}
